@@ -58,6 +58,8 @@
 //! - Special keys like backspace and CTL+C will not be processed by terminal driver
 //! - New line character will not be processed therefore `println!` can't be used, use `write!` instead
 //!
+//! Raw mode can be enabled/disabled with the [enable_raw_mode](terminal::enable_raw_mode) and [disable_raw_mode](terminal::disable_raw_mode) functions.
+//!
 //! ## Examples
 //!
 //! ```no_run
@@ -96,6 +98,21 @@ use crate::{csi, impl_display, Result};
 
 pub(crate) mod sys;
 
+/// Tells whether the raw mode is enabled.
+///
+/// Please have a look at the [raw mode](./#raw-mode) section.
+pub fn is_raw_mode_enabled() -> Result<bool> {
+    #[cfg(unix)]
+    {
+        Ok(sys::is_raw_mode_enabled())
+    }
+
+    #[cfg(windows)]
+    {
+        sys::is_raw_mode_enabled()
+    }
+}
+
 /// Enables raw mode.
 ///
 /// Please have a look at the [raw mode](./#raw-mode) section.
@@ -237,6 +254,8 @@ impl Command for LeaveAlternateScreen {
 pub enum ClearType {
     /// All cells.
     All,
+    /// All plus history
+    Purge,
     /// All cells from the cursor position downwards.
     FromCursorDown,
     /// All cells from the cursor position upwards.
@@ -305,6 +324,7 @@ impl Command for Clear {
     fn write_ansi(&self, f: &mut impl fmt::Write) -> fmt::Result {
         f.write_str(match self.0 {
             ClearType::All => csi!("2J"),
+            ClearType::Purge => csi!("3J"),
             ClearType::FromCursorDown => csi!("J"),
             ClearType::FromCursorUp => csi!("1J"),
             ClearType::CurrentLine => csi!("2K"),
@@ -367,7 +387,7 @@ mod tests {
 
     use crate::execute;
 
-    use super::{size, SetSize};
+    use super::*;
 
     // Test is disabled, because it's failing on Travis CI
     #[test]
@@ -390,4 +410,29 @@ mod tests {
 
         assert_eq!((width, height), size().unwrap());
     }
+
+    #[test]
+    fn test_raw_mode() {
+        // check we start from normal mode (may fail on some test harnesses)
+        assert_eq!(is_raw_mode_enabled().unwrap(), false);
+
+        // enable the raw mode
+        enable_raw_mode().unwrap();
+
+        // check it worked (on unix it doesn't really check the underlying
+        // tty but rather check that the code is consistent)
+        assert_eq!(is_raw_mode_enabled().unwrap(), true);
+
+        // enable it again, this should not change anything
+        enable_raw_mode().unwrap();
+
+        // check we're still in raw mode
+        assert_eq!(is_raw_mode_enabled().unwrap(), true);
+
+        // now let's disable it
+        disable_raw_mode().unwrap();
+
+        // check we're back to normal mode
+        assert_eq!(is_raw_mode_enabled().unwrap(), false);
+    }
 }
